@@ -12,10 +12,21 @@ from random import randint
 
 is_py3k = bool(sys.version_info[0] == 3)
 
+# imports needed inside tests
+import copy
+import pickle
+import itertools
+
+try:
+    import shelve, hashlib
+except ImportError:
+    shelve = hashlib = None
+
 if is_py3k:
     from io import StringIO
 else:
     from cStringIO import StringIO
+    import cPickle
 
 
 from bitarray import bitarray, bitdiff, bits2bytes, __version__
@@ -23,18 +34,19 @@ from bitarray import bitarray, bitdiff, bits2bytes, __version__
 
 tests = []
 
-if sys.version_info[:2] < (2, 6):
-    def next(x):
-        return x.next()
-
 
 def to_bytes(s):
     if is_py3k:
         return bytes(s.encode('latin1'))
-    elif sys.version_info[:2] >= (2, 6):
-        return bytes(s)
     else:
-        return s
+        return bytes(s)  # which is str for Python 2
+
+
+if is_py3k:
+    def unicode(*args):
+        if len(args) == 0:
+            return ''
+        return args[0]
 
 
 class Util(object):
@@ -185,11 +197,13 @@ class CreateObjectTests(unittest.TestCase, Util):
         a = bitarray(endian='little')
         a.fromstring('A')
         self.assertEqual(a.endian(), 'little')
+        self.assert_(isinstance(a.endian(), str))
         self.check_obj(a)
 
         b = bitarray(endian='big')
         b.fromstring('A')
         self.assertEqual(b.endian(), 'big')
+        self.assert_(isinstance(a.endian(), str))
         self.check_obj(b)
 
         self.assertEqual(a.tostring(), b.tostring())
@@ -605,6 +619,15 @@ class MiscTests(unittest.TestCase, Util):
         self.assertEqual(bool(bitarray('0')), True)
         self.assertEqual(bool(bitarray('1')), True)
 
+    def test_to01(self):
+        a = bitarray()
+        self.assertEqual(a.to01(), '')
+        self.assert_(isinstance(a.to01(), str))
+
+        a = bitarray('101')
+        self.assertEqual(a.to01(), '101')
+        self.assert_(isinstance(a.to01(), str))
+
     def test_iterate(self):
         for lst in self.randomlists():
             acc = []
@@ -720,10 +743,9 @@ class MiscTests(unittest.TestCase, Util):
         self.assertEqual(a.tobytes(), b.tobytes())
 
     def test_pickle(self):
-        from pickle import loads, dumps
         for v in range(3):
             for a in self.randombitarrays():
-                b = loads(dumps(a, v))
+                b = pickle.loads(pickle.dumps(a, v))
                 self.assert_(b is not a)
                 self.assertEQUAL(a, b)
 
@@ -731,16 +753,13 @@ class MiscTests(unittest.TestCase, Util):
         if is_py3k:
             return
         for v in range(3):
-            from cPickle import loads, dumps
             for a in self.randombitarrays():
-                b = loads(dumps(a, v))
+                b = cPickle.loads(cPickle.dumps(a, v))
                 self.assert_(b is not a)
                 self.assertEQUAL(a, b)
 
     def test_overflow(self):
-        from platform import architecture
-
-        if architecture()[0] == '64bit':
+        if tuple.__itemsize__ == 8:
             return
 
         self.assertRaises(OverflowError, bitarray.__new__,
@@ -749,6 +768,30 @@ class MiscTests(unittest.TestCase, Util):
         a = bitarray(10 ** 6)
         self.assertRaises(OverflowError, a.__imul__, 17180)
 
+    def test_unicode1(self):
+        a = bitarray(unicode())
+        self.assertEqual(a, bitarray())
+
+        a = bitarray(unicode('111001'))
+        self.assertEqual(a, bitarray('111001'))
+
+        for a in self.randombitarrays():
+            b = bitarray(unicode(a.to01()))
+            self.assertEqual(a, b)
+
+    def test_unicode2(self):
+        a = bitarray()
+        a.extend(unicode())
+        self.assertEqual(a, bitarray())
+
+        a = bitarray()
+        a.extend(unicode('001011'))
+        self.assertEqual(a, bitarray('001011'))
+
+        for a in self.randombitarrays():
+            b = bitarray()
+            b.extend(unicode(a.to01()))
+            self.assertEqual(a, b)
 
 tests.append(MiscTests)
 
@@ -760,9 +803,6 @@ class SpecialMethodTests(unittest.TestCase, Util):
         a = bitarray()
         self.assertTrue(a.all())
 
-        if sys.version_info[:2] < (2, 5):
-            return
-
         for a in self.randombitarrays():
             self.assertEqual(all(a),          a.all())
             self.assertEqual(all(a.tolist()), a.all())
@@ -772,20 +812,19 @@ class SpecialMethodTests(unittest.TestCase, Util):
         a = bitarray()
         self.assertFalse(a.any())
 
-        if sys.version_info[:2] < (2, 5):
-            return
-
         for a in self.randombitarrays():
             self.assertEqual(any(a),          a.any())
             self.assertEqual(any(a.tolist()), a.any())
 
 
     def test_repr(self):
-        a = bitarray()
-        self.assertEqual(repr(a), "bitarray()")
+        r = repr(bitarray())
+        self.assertEqual(r, "bitarray()")
+        self.assert_(isinstance(r, str))
 
-        a = bitarray('10111')
-        self.assertEqual(repr(a), "bitarray('10111')")
+        r = repr(bitarray('10111'))
+        self.assertEqual(r, "bitarray('10111')")
+        self.assert_(isinstance(r, str))
 
         for a in self.randombitarrays():
             b = eval(repr(a))
@@ -795,7 +834,6 @@ class SpecialMethodTests(unittest.TestCase, Util):
 
 
     def test_copy(self):
-        import copy
         for a in self.randombitarrays():
             b = a.copy()
             self.assert_(b is not a)
@@ -1189,10 +1227,6 @@ class ExtendTests(unittest.TestCase, Util):
                 self.check_obj(c)
 
     def test_iterator2(self):
-        try:
-            import itertools
-        except ImportError:
-            return
         a = bitarray()
         a.extend(itertools.repeat(True, 23))
         self.assertEqual(a, bitarray(23 * '1'))
@@ -1283,6 +1317,11 @@ class MethodTests(unittest.TestCase, Util):
         self.assertRaises(ValueError, a.index, True)
         a[20] = a[27] = 1
         self.assertEqual(a.index(42), 20)
+        self.assertEqual(a.index(1, 21), 27)
+        self.assertEqual(a.index(1, 27), 27)
+        self.assertRaises(ValueError, a.index, 1, 5, 17)
+        self.assertRaises(ValueError, a.index, 1, 21, 27)
+        self.assertRaises(ValueError, a.index, 1, 28)
         self.assertEqual(a.index(0), 0)
 
         a = bitarray(200 * [True])
@@ -1304,7 +1343,7 @@ class MethodTests(unittest.TestCase, Util):
                 a[m] = 0
                 self.assertEqual(a.index(0), m)
 
-    def test_index2(self):
+    def test_index2a(self):
         a = bitarray('00001000' '00000000' '0010000')
         self.assertEqual(a.index(1), 4)
         self.assertEqual(a.index(1, 1), 4)
@@ -1313,6 +1352,15 @@ class MethodTests(unittest.TestCase, Util):
         self.assertRaises(ValueError, a.index, 1, 5, 18)
         self.assertRaises(ValueError, a.index, 1, 19)
 
+    def test_index2b(self):
+        a = bitarray('11110111' '11111111' '1101111')
+        self.assertEqual(a.index(0), 4)
+        self.assertEqual(a.index(0, 1), 4)
+        self.assertEqual(a.index(1, 4), 5)
+        self.assertEqual(a.index(0, 5), 18)
+        self.assertRaises(ValueError, a.index, 0, 5, 18)
+        self.assertRaises(ValueError, a.index, 0, 19)
+
     def test_index3(self):
         a = bitarray(2000)
         a.setall(0)
@@ -1332,6 +1380,27 @@ class MethodTests(unittest.TestCase, Util):
                 res2 = None
             self.assertEqual(res1, res2)
 
+    def test_index4(self):
+        for n in range(1, 50):
+            a = bitarray(n)
+            i = randint(0, 1)
+            a.setall(i)
+            for unused in range(randint(1, 4)):
+                a[randint(0, n-1)] = 1-i
+            aa = a.tolist()
+            for unused in range(100):
+                start = randint(-50, n+50)
+                stop = randint(-50, n+50)
+                try:
+                    res1 = a.index(1-i, start, stop)
+                except ValueError:
+                    res1 = None
+                try:
+                    res2 = aa.index(1-i, start, stop)
+                except ValueError:
+                    res2 = None
+                self.assertEqual(res1, res2)
+
 
     def test_count(self):
         a = bitarray('10011')
@@ -1406,6 +1475,17 @@ class MethodTests(unittest.TestCase, Util):
             self.assertEqual(list(a.itersearch(b)), res)
             self.assertEqual([p for p in a.itersearch(b)], res)
 
+    def test_search4(self):
+        for a in self.randombitarrays():
+            aa = a.to01()
+            for sub in '0', '1', '01', '01', '11', '101', '1111111':
+                sr = a.search(bitarray(sub), 1)
+                try:
+                    p = sr[0]
+                except IndexError:
+                    p = -1
+                self.assertEqual(p, aa.find(sub))
+
     def test_search_type(self):
         a = bitarray('10011')
         it = a.itersearch(bitarray('1'))
@@ -1663,6 +1743,10 @@ class StringTests(unittest.TestCase, Util):
 
     def test_unpack(self):
         a = bitarray('01')
+        if is_py3k:
+            self.assert_(isinstance(a.unpack(), bytes))
+        else:
+            self.assert_(isinstance(a.unpack(), str))
         self.assertEqual(a.unpack(), to_bytes('\x00\xff'))
         self.assertEqual(a.unpack(to_bytes('A')), to_bytes('A\xff'))
         self.assertEqual(a.unpack(to_bytes('0'), to_bytes('1')),
@@ -1728,35 +1812,30 @@ class FileTests(unittest.TestCase, Util):
 
 
     def test_pickle(self):
-        from pickle import load, dump
-
         for v in range(3):
             for a in self.randombitarrays():
                 fo = open(self.tmpfname, 'wb')
-                dump(a, fo, v)
+                pickle.dump(a, fo, v)
                 fo.close()
-                b = load(open(self.tmpfname, 'rb'))
+                b = pickle.load(open(self.tmpfname, 'rb'))
                 self.assert_(b is not a)
                 self.assertEQUAL(a, b)
 
     def test_cPickle(self):
         if is_py3k:
             return
-        from cPickle import load, dump
-
         for v in range(3):
             for a in self.randombitarrays():
                 fo = open(self.tmpfname, 'wb')
-                dump(a, fo, v)
+                cPickle.dump(a, fo, v)
                 fo.close()
-                b = load(open(self.tmpfname, 'rb'))
+                b = cPickle.load(open(self.tmpfname, 'rb'))
                 self.assert_(b is not a)
                 self.assertEQUAL(a, b)
 
     def test_shelve(self):
-        if sys.version_info[:2] < (2, 5):
+        if not shelve or hasattr(sys, 'gettotalrefcount'):
             return
-        import shelve, hashlib
 
         d = shelve.open(self.tmpfname)
         stored = []
@@ -1841,6 +1920,7 @@ class FileTests(unittest.TestCase, Util):
 
         b = bitarray()
         f = open(self.tmpfname, 'rb')
+        b.fromfile(f, 0);     self.assertEqual(b.tostring(), '')
         b.fromfile(f, 1);     self.assertEqual(b.tostring(), 'A')
         f.read(1)
         b = bitarray()
